from yafowil.base import factory


DOC_ACE_PYTHON = """
ACE in Python mode
------------------

.. code-block:: python

    value = "def foo():\\n    return 'foo'"
    ace = factory('#field:ace', value=value, props={
        'label': 'Python source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'python'})
"""

def ace_python():
    part = factory(u'fieldset', name='yafowil.widget.ace.python')
    value = "def foo():\n    return 'foo'"
    part['ace'] = factory('#field:ace', value=value, props={
        'label': 'Python source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'python'})
    return {'widget': part,
            'doc': DOC_ACE_PYTHON,
            'title': 'Python'}


DOC_ACE_JS = """
ACE in Javascript mode
----------------------

.. code-block:: python

    value = "var foo = function() {\\n    return 'foo';\\n};"
    ace = factory('#field:ace', value=value, props={
        'label': 'Javascript Source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'javascript'})

"""

def ace_js():
    part = factory(u'fieldset', name='yafowil.widget.ace.js')
    value = "var foo = function() {\n    return 'foo';\n};"
    part['ace'] = factory('#field:ace', value=value, props={
        'label': 'Javascript Source',
        'required': 'Code is required',
        'theme': 'github',
        'mode': 'javascript'})
    return {'widget': part,
            'doc': DOC_ACE_JS,
            'title': 'Javascript'}


def get_example():
    return [ace_python(), ace_js()]



from yafowil.base import (
    factory,
    fetch_value,
)
from yafowil.common import (
    generic_extractor,
    generic_required_extractor,
)
from yafowil.utils import (
    cssid,
    managedprops,
)


@managedprops('theme', 'mode')
def ace_edit_renderer(widget, data):
    value = fetch_value(widget, data)
    if not value:
        value = ''
    ta_attrs = {
        'id': cssid(widget, 'ace', 'value'),
        'name': widget.dottedpath,
        'class': 'ace-editor-value',
        'style': 'display:none;',
    }
    ta = data.tag('textarea', value, **ta_attrs)
    editor_attrs = {
        'id': cssid(widget, 'ace'),
        'class': 'ace-editor',
    }
    editor = data.tag('div', value, **editor_attrs)
    wrapper_css = [
        'ace-editor-wrapper',
        'ace-option-theme-%s' % widget.attrs['theme'],
        'ace-option-mode-%s' % widget.attrs['mode'],
    ]
    wrapper_attrs = {'class': ' '.join(wrapper_css)}
    return data.tag('div', ta + editor, **wrapper_attrs)


def ace_display_renderer(widget, data):
    raise NotImplementedError(u"``yafowil.widget.ace`` does not support "
                              u"display mode yet")


factory.register(
    'ace',
    extractors=[generic_extractor, generic_required_extractor],
    edit_renderers=[ace_edit_renderer],
    display_renderers=[ace_display_renderer])

factory.doc['blueprint']['ace'] = \
"""Add-on blueprint `yafowil.widget.ace 
<http://github.com/bluedynamics/yafowil.widget.ace/>`_ .
"""

factory.defaults['ace.default'] = ''

factory.defaults['ace.class'] = 'ace_editor'

factory.defaults['ace.theme'] = 'github'
factory.doc['props']['ace.theme'] = \
"""ACE Theme.
"""

factory.defaults['ace.mode'] = 'python'
factory.doc['props']['ace.mode'] = \
"""ACE Mode.
"""
